// Persistence of Vision Ray Tracer Scene Description File
// File: ExtrScen.inc
// Vers: 3.1
// Desc: Testscene for Extruder.inc
// Date: 1999-01-22
// Auth: ingo
//

#version 3.1;

#include "extruder.inc"

global_settings {assumed_gamma 1.0}

light_source {
  < 500, 500,-500>
  rgb 1
}

camera {
  location  < 0, 1,-3>
  look_at   < 0, 1, 0>
  angle 75
}

/*------Genaral spline description:
  #declare Spline = array[n][4]{
    {P1      , P2, P3, P4},
    {P2+P2-P3, P3, P5, P6},
    {P3+P3-P5, P5, P7, P8}
  }
  
  Each row is a spline segment,
  collum1 contains controle points
  collum2 contains start point of segment
  collum3 contains end point of segment
  collum4 contains controle points
  Shape splines can be open and closed.
  Path splines can be open and closed.
*/


// Declare the shape_splines

#declare One_Shape = array[2][4] {        //Circle
  {<-1, 0,-1>, <-1, 0, 0>, < 1, 0, 0>, < 1, 0,-1>}
  {< 1, 0, 1>, < 1, 0, 0>, <-1, 0, 0>, <-1, 0, 1>}  
}

#declare Two_Shape = array[4][4] {        //Square
  {<-1, 0, 0>, <-1, 0, 0>, < 0, 0, 1>, < 0, 0, 1>}
  {< 0, 0, 1>, < 0, 0, 1>, < 1, 0, 0>, < 1, 0, 0>}
  {< 1, 0, 0>, < 1, 0, 0>, < 0, 0,-1>, < 0, 0,-1>}
  {< 0, 0,-1>, < 0, 0,-1>, <-1, 0, 0>, <-1, 0, 0>}
}


// Declare the path_spline: straight, two segments

#declare Path= array[2][4] {                        
  {< 0, 0, 0>, < 0, 0, 0>, < 0, 1, 0>, < 0, 1, 0>}
  {< 0, 1, 0>, < 0, 1, 0>, < 0, 2, 0>, < 0, 2, 0>}
}

// Declare the shape order along the path
// The shape goes from circle to square to circle
#declare Order= array[3] {
  One_Shape,
  Two_Shape,
  One_Shape
}

object {
  Extrude(Path, Order, 30, 50) 
  rotate <0,-80,0>
  pigment {rgb 1}
}

/* 
   Extrude(Path_Spline, Shape_Order, PathStep, ShapeStep)
   Decreasing the PathStep makes the oject smaller
   Decreasing the ShapeStep makes the gap bigger
   This should not be so! Problem is how to make the last
   triangles on a path or shape segment????
*/