// Persistence of Vision Ray Tracer Scene Description File
// File: Extruder.inc
// Vers: 3.1
// Desc: Macro's for extruding a spline-shape along a spline-path
// Date: 1999-01-20
// Auth: ingo, using ideas and code by Chris Colefax & Gilles Tran
//


//------This macro calculates a point on a spline defined in an array
#macro Sp_Pos(Spline_Array, Step)

  #declare _SP_Step = Step;

  #declare spline_segments= dimension_size(Spline_Array, 1);  // Amount of segments in array.
  #declare _SP_curseg= ceil(spline_segments*_SP_Step);        // Find needed segment.
  #if (_SP_curseg<=0) #declare _SP_curseg=1; #end
  #declare _SP_curseg=_SP_curseg-1;                           // NOT HAPPY WITH THIS!
                   
  #declare point0= Spline_Array[_SP_curseg][0];               // Get points from that segment.
  #declare point1= Spline_Array[_SP_curseg][1];
  #declare point2= Spline_Array[_SP_curseg][2];
  #declare point3= Spline_Array[_SP_curseg][3];

  #if (_SP_Step = 1 & _SP_curseg = spline_segments)           // Calculate spline position.
    #declare spline_pos = point2;
  #else
    #declare _SP_Step  = mod (_SP_Step * spline_segments, 1);
    #declare spline_pos =
      pow(1 - _SP_Step, 3) *     point1 +
      pow(1 - _SP_Step, 2) * 3 * _SP_Step         * (point1 + point1 - point0) +
      (1 - _SP_Step)       * 3 * pow(_SP_Step, 2) * (point2 + point2 - point3) +
      pow(_SP_Step, 3)     *     point2;
  #end //if
  //#debug concat("\nTaDaa = ",str(spline_pos.y,1,1),"\n")

#end //macro




//------This macro generates a mesh out of splines

#macro Extrude(Path_Spline, Shape_Order, PathStep, ShapeStep)

  #declare iStep= 1/PathStep;         // step size for the path
  #declare iSegStep= 1/ShapeStep;     // step size for the shapes

mesh{

  #declare iPath=0;
  #while (iPath != 1-(iStep+iStep))   // loop on path  NOT HAPPY WITH THIS, PROBLEM AREA
    #declare iPath2= iPath+iStep;
    #declare iPath3= iPath2+iStep;

    Sp_Pos(Path_Spline, iPath)        // present point on the path
    #declare P1= spline_pos;

    Sp_Pos(Path_Spline, iPath2)       // next point on the path PROBLEM AREA
    #declare P2=  spline_pos;         

    Sp_Pos(Path_Spline, iPath3)       // next next point on the path PROBLEM AREA
    #declare P3= spline_pos;
    
    // vector calculation to rotate the shape along the P2-P1 vector (thanks John Van Sickle !)
    #declare yV1= vnormalize(P2-P1);
    #declare xV1= vnormalize(vcross(yV1,z));
    #declare zV1= vcross(xV1,yV1);

    // vector calculation to rotate the shape along the P3-P2 vector
    #declare yV2= vnormalize(P3-P2);
    #declare xV2= vnormalize(vcross(yV2,z));
    #declare zV2= vcross(xV2,yV2);

// shape definition
    #declare iSeg=0;
    #while (iSeg != 1-iSegStep)       // NOT HAPPY WITH THIS
      #declare iSeg2=iSeg+iSegStep;   // PROBLEM
    
      #declare spline_segments= dimension_size (Path_Spline, 1);  // Amount of segments in array.
      #declare Curseg= ceil(spline_segments*iPath);               // Find needed segment.
      #if (Curseg<=0) #declare Curseg=1; #end               

      #declare StartShape= Shape_Order[Curseg-1]
      #declare EndShape= Shape_Order[Curseg]                      // This does not feel right

      Sp_Pos(StartShape, iSeg)        // First point on start-shape  
      #declare aP1= spline_pos;
      Sp_Pos(EndShape, iSeg)          // First point on end-shape    
      #declare aP2= spline_pos;
      #declare sP1= aP1+(aP2-aP1)*iPath;         // interpolate                               
      #declare sP2= aP1+(aP2-aP1)*iPath2;                                       
                                                                             
      Sp_Pos(StartShape, iSeg2)       // Second point on first shape  PROBLEM
      #declare aP3= spline_pos;
      Sp_Pos(EndShape, iSeg2)         // Second point on end shape    PROBLEM
      #declare aP4= spline_pos;
      #declare sP3= aP3+(aP4-aP3)*iPath;         // interpolate
      #declare sP4= aP3+(aP4-aP3)*iPath2;        

      #declare x1= sP1.x;
      #declare y1= sP1.y;
      #declare z1= sP1.z;
    
      #declare x2= sP2.x;
      #declare y2= sP1.y;
      #declare z2= sP2.z;

      #declare x1s= sP3.x;
      #declare y1s= sP3.y;
      #declare z1s= sP3.z;
    
      #declare x2s= sP4.x;
      #declare y2s= sP4.y;
      #declare z2s= sP4.z;


// rotates the 4 points that will make the 2 triangles
      #declare posV1= <x1*xV1.x+y1*yV1.x+z1*zV1.x+P1.x, 
                       x1*xV1.y+y1*yV1.y+z1*zV1.y+P1.y, 
                       x1*xV1.z+y1*yV1.z+z1*zV1.z+P1.z>;
    
      #declare posV2= <x2*xV2.x+y2*yV2.x+z2*zV2.x+P2.x, 
                       x2*xV2.y+y2*yV2.y+z2*zV2.y+P2.y, 
                       x2*xV2.z+y2*yV2.z+z2*zV2.z+P2.z>;
    
      #declare posV1s= <x1s*xV1.x+y1s*yV1.x+z1s*zV1.x+P1.x, 
                        x1s*xV1.y+y1s*yV1.y+z1s*zV1.y+P1.y, 
                        x1s*xV1.z+y1s*yV1.z+z1s*zV1.z+P1.z>;
    
      #declare posV2s= <x2s*xV2.x+y2s*yV2.x+z2s*zV2.x+P2.x, 
                        x2s*xV2.y+y2s*yV2.y+z2s*zV2.y+P2.y, 
                        x2s*xV2.z+y2s*yV2.z+z2s*zV2.z+P2.z>;
    
      triangle{posV1,posV2,posV1s}
      triangle{posV1s,posV2,posV2s}

      #declare iSeg= iSeg + iSegStep; // next point on the shape
    #end //while

    #declare iPath=iPath2; // next point on the path
  #end //while
}

#end //macro


